# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx._utils import parse_datetime
from telnyx.types.messaging_tollfree.verification import (
    RequestListResponse,
    VerificationRequestEgress,
    VerificationRequestStatus,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestRequests:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        request = client.messaging_tollfree.verification.requests.create(
            additional_information="additionalInformation",
            business_addr1="600 Congress Avenue",
            business_city="Austin",
            business_contact_email="email@example.com",
            business_contact_first_name="John",
            business_contact_last_name="Doe",
            business_contact_phone="+18005550100",
            business_name="Telnyx LLC",
            business_state="Texas",
            business_zip="78701",
            corporate_website="http://example.com",
            isv_reseller="isvReseller",
            message_volume="100,000",
            opt_in_workflow="User signs into the Telnyx portal, enters a number and is prompted to select whether they want to use 2FA verification for security purposes. If they've opted in a confirmation message is sent out to the handset",
            opt_in_workflow_image_urls=[
                {"url": "https://telnyx.com/sign-up"},
                {"url": "https://telnyx.com/company/data-privacy"},
            ],
            phone_numbers=[{"phone_number": "+18773554398"}, {"phone_number": "+18773554399"}],
            production_message_content="Your Telnyx OTP is XXXX",
            use_case="2FA",
            use_case_summary="This is a use case where Telnyx sends out 2FA codes to portal users to verify their identity in order to sign into the portal",
        )
        assert_matches_type(VerificationRequestEgress, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        request = client.messaging_tollfree.verification.requests.create(
            additional_information="additionalInformation",
            business_addr1="600 Congress Avenue",
            business_city="Austin",
            business_contact_email="email@example.com",
            business_contact_first_name="John",
            business_contact_last_name="Doe",
            business_contact_phone="+18005550100",
            business_name="Telnyx LLC",
            business_state="Texas",
            business_zip="78701",
            corporate_website="http://example.com",
            isv_reseller="isvReseller",
            message_volume="100,000",
            opt_in_workflow="User signs into the Telnyx portal, enters a number and is prompted to select whether they want to use 2FA verification for security purposes. If they've opted in a confirmation message is sent out to the handset",
            opt_in_workflow_image_urls=[
                {"url": "https://telnyx.com/sign-up"},
                {"url": "https://telnyx.com/company/data-privacy"},
            ],
            phone_numbers=[{"phone_number": "+18773554398"}, {"phone_number": "+18773554399"}],
            production_message_content="Your Telnyx OTP is XXXX",
            use_case="2FA",
            use_case_summary="This is a use case where Telnyx sends out 2FA codes to portal users to verify their identity in order to sign into the portal",
            age_gated_content=True,
            business_addr2="14th Floor",
            business_registration_country="US",
            business_registration_number="12-3456789",
            business_registration_type="EIN",
            doing_business_as="Acme Services",
            entity_type="SOLE_PROPRIETOR",
            help_message_response="Reply HELP for assistance or STOP to unsubscribe. Contact: support@example.com",
            opt_in_confirmation_response="You have successfully opted in to receive messages from Acme Corp",
            opt_in_keywords="START, YES, SUBSCRIBE",
            privacy_policy_url="https://example.com/privacy",
            terms_and_condition_url="https://example.com/terms",
            webhook_url="http://example-webhook.com",
        )
        assert_matches_type(VerificationRequestEgress, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.messaging_tollfree.verification.requests.with_raw_response.create(
            additional_information="additionalInformation",
            business_addr1="600 Congress Avenue",
            business_city="Austin",
            business_contact_email="email@example.com",
            business_contact_first_name="John",
            business_contact_last_name="Doe",
            business_contact_phone="+18005550100",
            business_name="Telnyx LLC",
            business_state="Texas",
            business_zip="78701",
            corporate_website="http://example.com",
            isv_reseller="isvReseller",
            message_volume="100,000",
            opt_in_workflow="User signs into the Telnyx portal, enters a number and is prompted to select whether they want to use 2FA verification for security purposes. If they've opted in a confirmation message is sent out to the handset",
            opt_in_workflow_image_urls=[
                {"url": "https://telnyx.com/sign-up"},
                {"url": "https://telnyx.com/company/data-privacy"},
            ],
            phone_numbers=[{"phone_number": "+18773554398"}, {"phone_number": "+18773554399"}],
            production_message_content="Your Telnyx OTP is XXXX",
            use_case="2FA",
            use_case_summary="This is a use case where Telnyx sends out 2FA codes to portal users to verify their identity in order to sign into the portal",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        request = response.parse()
        assert_matches_type(VerificationRequestEgress, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.messaging_tollfree.verification.requests.with_streaming_response.create(
            additional_information="additionalInformation",
            business_addr1="600 Congress Avenue",
            business_city="Austin",
            business_contact_email="email@example.com",
            business_contact_first_name="John",
            business_contact_last_name="Doe",
            business_contact_phone="+18005550100",
            business_name="Telnyx LLC",
            business_state="Texas",
            business_zip="78701",
            corporate_website="http://example.com",
            isv_reseller="isvReseller",
            message_volume="100,000",
            opt_in_workflow="User signs into the Telnyx portal, enters a number and is prompted to select whether they want to use 2FA verification for security purposes. If they've opted in a confirmation message is sent out to the handset",
            opt_in_workflow_image_urls=[
                {"url": "https://telnyx.com/sign-up"},
                {"url": "https://telnyx.com/company/data-privacy"},
            ],
            phone_numbers=[{"phone_number": "+18773554398"}, {"phone_number": "+18773554399"}],
            production_message_content="Your Telnyx OTP is XXXX",
            use_case="2FA",
            use_case_summary="This is a use case where Telnyx sends out 2FA codes to portal users to verify their identity in order to sign into the portal",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            request = response.parse()
            assert_matches_type(VerificationRequestEgress, request, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        request = client.messaging_tollfree.verification.requests.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(VerificationRequestStatus, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.messaging_tollfree.verification.requests.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        request = response.parse()
        assert_matches_type(VerificationRequestStatus, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.messaging_tollfree.verification.requests.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            request = response.parse()
            assert_matches_type(VerificationRequestStatus, request, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.messaging_tollfree.verification.requests.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        request = client.messaging_tollfree.verification.requests.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            additional_information="additionalInformation",
            business_addr1="600 Congress Avenue",
            business_city="Austin",
            business_contact_email="email@example.com",
            business_contact_first_name="John",
            business_contact_last_name="Doe",
            business_contact_phone="+18005550100",
            business_name="Telnyx LLC",
            business_state="Texas",
            business_zip="78701",
            corporate_website="http://example.com",
            isv_reseller="isvReseller",
            message_volume="100,000",
            opt_in_workflow="User signs into the Telnyx portal, enters a number and is prompted to select whether they want to use 2FA verification for security purposes. If they've opted in a confirmation message is sent out to the handset",
            opt_in_workflow_image_urls=[
                {"url": "https://telnyx.com/sign-up"},
                {"url": "https://telnyx.com/company/data-privacy"},
            ],
            phone_numbers=[{"phone_number": "+18773554398"}, {"phone_number": "+18773554399"}],
            production_message_content="Your Telnyx OTP is XXXX",
            use_case="2FA",
            use_case_summary="This is a use case where Telnyx sends out 2FA codes to portal users to verify their identity in order to sign into the portal",
        )
        assert_matches_type(VerificationRequestEgress, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        request = client.messaging_tollfree.verification.requests.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            additional_information="additionalInformation",
            business_addr1="600 Congress Avenue",
            business_city="Austin",
            business_contact_email="email@example.com",
            business_contact_first_name="John",
            business_contact_last_name="Doe",
            business_contact_phone="+18005550100",
            business_name="Telnyx LLC",
            business_state="Texas",
            business_zip="78701",
            corporate_website="http://example.com",
            isv_reseller="isvReseller",
            message_volume="100,000",
            opt_in_workflow="User signs into the Telnyx portal, enters a number and is prompted to select whether they want to use 2FA verification for security purposes. If they've opted in a confirmation message is sent out to the handset",
            opt_in_workflow_image_urls=[
                {"url": "https://telnyx.com/sign-up"},
                {"url": "https://telnyx.com/company/data-privacy"},
            ],
            phone_numbers=[{"phone_number": "+18773554398"}, {"phone_number": "+18773554399"}],
            production_message_content="Your Telnyx OTP is XXXX",
            use_case="2FA",
            use_case_summary="This is a use case where Telnyx sends out 2FA codes to portal users to verify their identity in order to sign into the portal",
            age_gated_content=True,
            business_addr2="14th Floor",
            business_registration_country="US",
            business_registration_number="12-3456789",
            business_registration_type="EIN",
            doing_business_as="Acme Services",
            entity_type="SOLE_PROPRIETOR",
            help_message_response="Reply HELP for assistance or STOP to unsubscribe. Contact: support@example.com",
            opt_in_confirmation_response="You have successfully opted in to receive messages from Acme Corp",
            opt_in_keywords="START, YES, SUBSCRIBE",
            privacy_policy_url="https://example.com/privacy",
            terms_and_condition_url="https://example.com/terms",
            webhook_url="http://example-webhook.com",
        )
        assert_matches_type(VerificationRequestEgress, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.messaging_tollfree.verification.requests.with_raw_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            additional_information="additionalInformation",
            business_addr1="600 Congress Avenue",
            business_city="Austin",
            business_contact_email="email@example.com",
            business_contact_first_name="John",
            business_contact_last_name="Doe",
            business_contact_phone="+18005550100",
            business_name="Telnyx LLC",
            business_state="Texas",
            business_zip="78701",
            corporate_website="http://example.com",
            isv_reseller="isvReseller",
            message_volume="100,000",
            opt_in_workflow="User signs into the Telnyx portal, enters a number and is prompted to select whether they want to use 2FA verification for security purposes. If they've opted in a confirmation message is sent out to the handset",
            opt_in_workflow_image_urls=[
                {"url": "https://telnyx.com/sign-up"},
                {"url": "https://telnyx.com/company/data-privacy"},
            ],
            phone_numbers=[{"phone_number": "+18773554398"}, {"phone_number": "+18773554399"}],
            production_message_content="Your Telnyx OTP is XXXX",
            use_case="2FA",
            use_case_summary="This is a use case where Telnyx sends out 2FA codes to portal users to verify their identity in order to sign into the portal",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        request = response.parse()
        assert_matches_type(VerificationRequestEgress, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.messaging_tollfree.verification.requests.with_streaming_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            additional_information="additionalInformation",
            business_addr1="600 Congress Avenue",
            business_city="Austin",
            business_contact_email="email@example.com",
            business_contact_first_name="John",
            business_contact_last_name="Doe",
            business_contact_phone="+18005550100",
            business_name="Telnyx LLC",
            business_state="Texas",
            business_zip="78701",
            corporate_website="http://example.com",
            isv_reseller="isvReseller",
            message_volume="100,000",
            opt_in_workflow="User signs into the Telnyx portal, enters a number and is prompted to select whether they want to use 2FA verification for security purposes. If they've opted in a confirmation message is sent out to the handset",
            opt_in_workflow_image_urls=[
                {"url": "https://telnyx.com/sign-up"},
                {"url": "https://telnyx.com/company/data-privacy"},
            ],
            phone_numbers=[{"phone_number": "+18773554398"}, {"phone_number": "+18773554399"}],
            production_message_content="Your Telnyx OTP is XXXX",
            use_case="2FA",
            use_case_summary="This is a use case where Telnyx sends out 2FA codes to portal users to verify their identity in order to sign into the portal",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            request = response.parse()
            assert_matches_type(VerificationRequestEgress, request, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.messaging_tollfree.verification.requests.with_raw_response.update(
                id="",
                additional_information="additionalInformation",
                business_addr1="600 Congress Avenue",
                business_city="Austin",
                business_contact_email="email@example.com",
                business_contact_first_name="John",
                business_contact_last_name="Doe",
                business_contact_phone="+18005550100",
                business_name="Telnyx LLC",
                business_state="Texas",
                business_zip="78701",
                corporate_website="http://example.com",
                isv_reseller="isvReseller",
                message_volume="100,000",
                opt_in_workflow="User signs into the Telnyx portal, enters a number and is prompted to select whether they want to use 2FA verification for security purposes. If they've opted in a confirmation message is sent out to the handset",
                opt_in_workflow_image_urls=[
                    {"url": "https://telnyx.com/sign-up"},
                    {"url": "https://telnyx.com/company/data-privacy"},
                ],
                phone_numbers=[{"phone_number": "+18773554398"}, {"phone_number": "+18773554399"}],
                production_message_content="Your Telnyx OTP is XXXX",
                use_case="2FA",
                use_case_summary="This is a use case where Telnyx sends out 2FA codes to portal users to verify their identity in order to sign into the portal",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        request = client.messaging_tollfree.verification.requests.list(
            page=1,
            page_size=1,
        )
        assert_matches_type(RequestListResponse, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        request = client.messaging_tollfree.verification.requests.list(
            page=1,
            page_size=1,
            date_end=parse_datetime("2019-12-27T18:11:19.117Z"),
            date_start=parse_datetime("2019-12-27T18:11:19.117Z"),
            phone_number="phone_number",
            status="Verified",
        )
        assert_matches_type(RequestListResponse, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.messaging_tollfree.verification.requests.with_raw_response.list(
            page=1,
            page_size=1,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        request = response.parse()
        assert_matches_type(RequestListResponse, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.messaging_tollfree.verification.requests.with_streaming_response.list(
            page=1,
            page_size=1,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            request = response.parse()
            assert_matches_type(RequestListResponse, request, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        request = client.messaging_tollfree.verification.requests.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(object, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.messaging_tollfree.verification.requests.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        request = response.parse()
        assert_matches_type(object, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.messaging_tollfree.verification.requests.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            request = response.parse()
            assert_matches_type(object, request, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.messaging_tollfree.verification.requests.with_raw_response.delete(
                "",
            )


class TestAsyncRequests:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        request = await async_client.messaging_tollfree.verification.requests.create(
            additional_information="additionalInformation",
            business_addr1="600 Congress Avenue",
            business_city="Austin",
            business_contact_email="email@example.com",
            business_contact_first_name="John",
            business_contact_last_name="Doe",
            business_contact_phone="+18005550100",
            business_name="Telnyx LLC",
            business_state="Texas",
            business_zip="78701",
            corporate_website="http://example.com",
            isv_reseller="isvReseller",
            message_volume="100,000",
            opt_in_workflow="User signs into the Telnyx portal, enters a number and is prompted to select whether they want to use 2FA verification for security purposes. If they've opted in a confirmation message is sent out to the handset",
            opt_in_workflow_image_urls=[
                {"url": "https://telnyx.com/sign-up"},
                {"url": "https://telnyx.com/company/data-privacy"},
            ],
            phone_numbers=[{"phone_number": "+18773554398"}, {"phone_number": "+18773554399"}],
            production_message_content="Your Telnyx OTP is XXXX",
            use_case="2FA",
            use_case_summary="This is a use case where Telnyx sends out 2FA codes to portal users to verify their identity in order to sign into the portal",
        )
        assert_matches_type(VerificationRequestEgress, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        request = await async_client.messaging_tollfree.verification.requests.create(
            additional_information="additionalInformation",
            business_addr1="600 Congress Avenue",
            business_city="Austin",
            business_contact_email="email@example.com",
            business_contact_first_name="John",
            business_contact_last_name="Doe",
            business_contact_phone="+18005550100",
            business_name="Telnyx LLC",
            business_state="Texas",
            business_zip="78701",
            corporate_website="http://example.com",
            isv_reseller="isvReseller",
            message_volume="100,000",
            opt_in_workflow="User signs into the Telnyx portal, enters a number and is prompted to select whether they want to use 2FA verification for security purposes. If they've opted in a confirmation message is sent out to the handset",
            opt_in_workflow_image_urls=[
                {"url": "https://telnyx.com/sign-up"},
                {"url": "https://telnyx.com/company/data-privacy"},
            ],
            phone_numbers=[{"phone_number": "+18773554398"}, {"phone_number": "+18773554399"}],
            production_message_content="Your Telnyx OTP is XXXX",
            use_case="2FA",
            use_case_summary="This is a use case where Telnyx sends out 2FA codes to portal users to verify their identity in order to sign into the portal",
            age_gated_content=True,
            business_addr2="14th Floor",
            business_registration_country="US",
            business_registration_number="12-3456789",
            business_registration_type="EIN",
            doing_business_as="Acme Services",
            entity_type="SOLE_PROPRIETOR",
            help_message_response="Reply HELP for assistance or STOP to unsubscribe. Contact: support@example.com",
            opt_in_confirmation_response="You have successfully opted in to receive messages from Acme Corp",
            opt_in_keywords="START, YES, SUBSCRIBE",
            privacy_policy_url="https://example.com/privacy",
            terms_and_condition_url="https://example.com/terms",
            webhook_url="http://example-webhook.com",
        )
        assert_matches_type(VerificationRequestEgress, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_tollfree.verification.requests.with_raw_response.create(
            additional_information="additionalInformation",
            business_addr1="600 Congress Avenue",
            business_city="Austin",
            business_contact_email="email@example.com",
            business_contact_first_name="John",
            business_contact_last_name="Doe",
            business_contact_phone="+18005550100",
            business_name="Telnyx LLC",
            business_state="Texas",
            business_zip="78701",
            corporate_website="http://example.com",
            isv_reseller="isvReseller",
            message_volume="100,000",
            opt_in_workflow="User signs into the Telnyx portal, enters a number and is prompted to select whether they want to use 2FA verification for security purposes. If they've opted in a confirmation message is sent out to the handset",
            opt_in_workflow_image_urls=[
                {"url": "https://telnyx.com/sign-up"},
                {"url": "https://telnyx.com/company/data-privacy"},
            ],
            phone_numbers=[{"phone_number": "+18773554398"}, {"phone_number": "+18773554399"}],
            production_message_content="Your Telnyx OTP is XXXX",
            use_case="2FA",
            use_case_summary="This is a use case where Telnyx sends out 2FA codes to portal users to verify their identity in order to sign into the portal",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        request = await response.parse()
        assert_matches_type(VerificationRequestEgress, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_tollfree.verification.requests.with_streaming_response.create(
            additional_information="additionalInformation",
            business_addr1="600 Congress Avenue",
            business_city="Austin",
            business_contact_email="email@example.com",
            business_contact_first_name="John",
            business_contact_last_name="Doe",
            business_contact_phone="+18005550100",
            business_name="Telnyx LLC",
            business_state="Texas",
            business_zip="78701",
            corporate_website="http://example.com",
            isv_reseller="isvReseller",
            message_volume="100,000",
            opt_in_workflow="User signs into the Telnyx portal, enters a number and is prompted to select whether they want to use 2FA verification for security purposes. If they've opted in a confirmation message is sent out to the handset",
            opt_in_workflow_image_urls=[
                {"url": "https://telnyx.com/sign-up"},
                {"url": "https://telnyx.com/company/data-privacy"},
            ],
            phone_numbers=[{"phone_number": "+18773554398"}, {"phone_number": "+18773554399"}],
            production_message_content="Your Telnyx OTP is XXXX",
            use_case="2FA",
            use_case_summary="This is a use case where Telnyx sends out 2FA codes to portal users to verify their identity in order to sign into the portal",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            request = await response.parse()
            assert_matches_type(VerificationRequestEgress, request, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        request = await async_client.messaging_tollfree.verification.requests.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(VerificationRequestStatus, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_tollfree.verification.requests.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        request = await response.parse()
        assert_matches_type(VerificationRequestStatus, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_tollfree.verification.requests.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            request = await response.parse()
            assert_matches_type(VerificationRequestStatus, request, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.messaging_tollfree.verification.requests.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        request = await async_client.messaging_tollfree.verification.requests.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            additional_information="additionalInformation",
            business_addr1="600 Congress Avenue",
            business_city="Austin",
            business_contact_email="email@example.com",
            business_contact_first_name="John",
            business_contact_last_name="Doe",
            business_contact_phone="+18005550100",
            business_name="Telnyx LLC",
            business_state="Texas",
            business_zip="78701",
            corporate_website="http://example.com",
            isv_reseller="isvReseller",
            message_volume="100,000",
            opt_in_workflow="User signs into the Telnyx portal, enters a number and is prompted to select whether they want to use 2FA verification for security purposes. If they've opted in a confirmation message is sent out to the handset",
            opt_in_workflow_image_urls=[
                {"url": "https://telnyx.com/sign-up"},
                {"url": "https://telnyx.com/company/data-privacy"},
            ],
            phone_numbers=[{"phone_number": "+18773554398"}, {"phone_number": "+18773554399"}],
            production_message_content="Your Telnyx OTP is XXXX",
            use_case="2FA",
            use_case_summary="This is a use case where Telnyx sends out 2FA codes to portal users to verify their identity in order to sign into the portal",
        )
        assert_matches_type(VerificationRequestEgress, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        request = await async_client.messaging_tollfree.verification.requests.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            additional_information="additionalInformation",
            business_addr1="600 Congress Avenue",
            business_city="Austin",
            business_contact_email="email@example.com",
            business_contact_first_name="John",
            business_contact_last_name="Doe",
            business_contact_phone="+18005550100",
            business_name="Telnyx LLC",
            business_state="Texas",
            business_zip="78701",
            corporate_website="http://example.com",
            isv_reseller="isvReseller",
            message_volume="100,000",
            opt_in_workflow="User signs into the Telnyx portal, enters a number and is prompted to select whether they want to use 2FA verification for security purposes. If they've opted in a confirmation message is sent out to the handset",
            opt_in_workflow_image_urls=[
                {"url": "https://telnyx.com/sign-up"},
                {"url": "https://telnyx.com/company/data-privacy"},
            ],
            phone_numbers=[{"phone_number": "+18773554398"}, {"phone_number": "+18773554399"}],
            production_message_content="Your Telnyx OTP is XXXX",
            use_case="2FA",
            use_case_summary="This is a use case where Telnyx sends out 2FA codes to portal users to verify their identity in order to sign into the portal",
            age_gated_content=True,
            business_addr2="14th Floor",
            business_registration_country="US",
            business_registration_number="12-3456789",
            business_registration_type="EIN",
            doing_business_as="Acme Services",
            entity_type="SOLE_PROPRIETOR",
            help_message_response="Reply HELP for assistance or STOP to unsubscribe. Contact: support@example.com",
            opt_in_confirmation_response="You have successfully opted in to receive messages from Acme Corp",
            opt_in_keywords="START, YES, SUBSCRIBE",
            privacy_policy_url="https://example.com/privacy",
            terms_and_condition_url="https://example.com/terms",
            webhook_url="http://example-webhook.com",
        )
        assert_matches_type(VerificationRequestEgress, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_tollfree.verification.requests.with_raw_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            additional_information="additionalInformation",
            business_addr1="600 Congress Avenue",
            business_city="Austin",
            business_contact_email="email@example.com",
            business_contact_first_name="John",
            business_contact_last_name="Doe",
            business_contact_phone="+18005550100",
            business_name="Telnyx LLC",
            business_state="Texas",
            business_zip="78701",
            corporate_website="http://example.com",
            isv_reseller="isvReseller",
            message_volume="100,000",
            opt_in_workflow="User signs into the Telnyx portal, enters a number and is prompted to select whether they want to use 2FA verification for security purposes. If they've opted in a confirmation message is sent out to the handset",
            opt_in_workflow_image_urls=[
                {"url": "https://telnyx.com/sign-up"},
                {"url": "https://telnyx.com/company/data-privacy"},
            ],
            phone_numbers=[{"phone_number": "+18773554398"}, {"phone_number": "+18773554399"}],
            production_message_content="Your Telnyx OTP is XXXX",
            use_case="2FA",
            use_case_summary="This is a use case where Telnyx sends out 2FA codes to portal users to verify their identity in order to sign into the portal",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        request = await response.parse()
        assert_matches_type(VerificationRequestEgress, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_tollfree.verification.requests.with_streaming_response.update(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            additional_information="additionalInformation",
            business_addr1="600 Congress Avenue",
            business_city="Austin",
            business_contact_email="email@example.com",
            business_contact_first_name="John",
            business_contact_last_name="Doe",
            business_contact_phone="+18005550100",
            business_name="Telnyx LLC",
            business_state="Texas",
            business_zip="78701",
            corporate_website="http://example.com",
            isv_reseller="isvReseller",
            message_volume="100,000",
            opt_in_workflow="User signs into the Telnyx portal, enters a number and is prompted to select whether they want to use 2FA verification for security purposes. If they've opted in a confirmation message is sent out to the handset",
            opt_in_workflow_image_urls=[
                {"url": "https://telnyx.com/sign-up"},
                {"url": "https://telnyx.com/company/data-privacy"},
            ],
            phone_numbers=[{"phone_number": "+18773554398"}, {"phone_number": "+18773554399"}],
            production_message_content="Your Telnyx OTP is XXXX",
            use_case="2FA",
            use_case_summary="This is a use case where Telnyx sends out 2FA codes to portal users to verify their identity in order to sign into the portal",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            request = await response.parse()
            assert_matches_type(VerificationRequestEgress, request, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.messaging_tollfree.verification.requests.with_raw_response.update(
                id="",
                additional_information="additionalInformation",
                business_addr1="600 Congress Avenue",
                business_city="Austin",
                business_contact_email="email@example.com",
                business_contact_first_name="John",
                business_contact_last_name="Doe",
                business_contact_phone="+18005550100",
                business_name="Telnyx LLC",
                business_state="Texas",
                business_zip="78701",
                corporate_website="http://example.com",
                isv_reseller="isvReseller",
                message_volume="100,000",
                opt_in_workflow="User signs into the Telnyx portal, enters a number and is prompted to select whether they want to use 2FA verification for security purposes. If they've opted in a confirmation message is sent out to the handset",
                opt_in_workflow_image_urls=[
                    {"url": "https://telnyx.com/sign-up"},
                    {"url": "https://telnyx.com/company/data-privacy"},
                ],
                phone_numbers=[{"phone_number": "+18773554398"}, {"phone_number": "+18773554399"}],
                production_message_content="Your Telnyx OTP is XXXX",
                use_case="2FA",
                use_case_summary="This is a use case where Telnyx sends out 2FA codes to portal users to verify their identity in order to sign into the portal",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        request = await async_client.messaging_tollfree.verification.requests.list(
            page=1,
            page_size=1,
        )
        assert_matches_type(RequestListResponse, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        request = await async_client.messaging_tollfree.verification.requests.list(
            page=1,
            page_size=1,
            date_end=parse_datetime("2019-12-27T18:11:19.117Z"),
            date_start=parse_datetime("2019-12-27T18:11:19.117Z"),
            phone_number="phone_number",
            status="Verified",
        )
        assert_matches_type(RequestListResponse, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_tollfree.verification.requests.with_raw_response.list(
            page=1,
            page_size=1,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        request = await response.parse()
        assert_matches_type(RequestListResponse, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_tollfree.verification.requests.with_streaming_response.list(
            page=1,
            page_size=1,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            request = await response.parse()
            assert_matches_type(RequestListResponse, request, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        request = await async_client.messaging_tollfree.verification.requests.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(object, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_tollfree.verification.requests.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        request = await response.parse()
        assert_matches_type(object, request, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_tollfree.verification.requests.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            request = await response.parse()
            assert_matches_type(object, request, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.messaging_tollfree.verification.requests.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["RecordingResponseData", "DownloadURLs"]


class DownloadURLs(BaseModel):
    mp3: Optional[str] = None
    """Link to download the recording in mp3 format."""

    wav: Optional[str] = None
    """Link to download the recording in wav format."""


class RecordingResponseData(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the recording."""

    call_control_id: Optional[str] = None
    """Unique identifier and token for controlling the call."""

    call_leg_id: Optional[str] = None
    """ID unique to the call leg (used to correlate webhook events)."""

    call_session_id: Optional[str] = None
    """
    ID that is unique to the call session and can be used to correlate webhook
    events. Call session is a group of related call legs that logically belong to
    the same phone call, e.g. an inbound and outbound leg of a transferred call.
    """

    channels: Optional[Literal["single", "dual"]] = None
    """
    When `dual`, the final audio file has the first leg on channel A, and the rest
    on channel B.
    """

    conference_id: Optional[str] = None
    """Uniquely identifies the conference."""

    created_at: Optional[str] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    download_urls: Optional[DownloadURLs] = None
    """Links to download the recording files."""

    duration_millis: Optional[int] = None
    """The duration of the recording in milliseconds."""

    record_type: Optional[Literal["recording"]] = None

    recording_ended_at: Optional[str] = None
    """ISO 8601 formatted date of when the recording ended."""

    recording_started_at: Optional[str] = None
    """ISO 8601 formatted date of when the recording started."""

    source: Optional[Literal["conference", "call"]] = None
    """The kind of event that led to this recording being created."""

    status: Optional[Literal["completed"]] = None
    """The status of the recording.

    Only `completed` recordings are currently supported.
    """

    updated_at: Optional[str] = None
    """ISO 8601 formatted date indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["MobileNetworkOperatorListParams", "Filter", "FilterName", "Page"]


class MobileNetworkOperatorListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter for mobile network operators (deepObject style).

    Originally: filter[name][starts_with], filter[name][contains],
    filter[name][ends_with], filter[country_code], filter[mcc], filter[mnc],
    filter[tadig], filter[network_preferences_enabled]
    """

    page: Page
    """Consolidated pagination parameter (deepObject style).

    Originally: page[number], page[size]
    """


class FilterName(TypedDict, total=False):
    contains: str
    """Filter by name containing match."""

    ends_with: str
    """Filter by name ending with."""

    starts_with: str
    """Filter by name starting with."""


class Filter(TypedDict, total=False):
    country_code: str
    """Filter by exact country_code."""

    mcc: str
    """Filter by exact MCC."""

    mnc: str
    """Filter by exact MNC."""

    name: FilterName
    """Advanced name filtering operations"""

    network_preferences_enabled: bool
    """Filter by network_preferences_enabled."""

    tadig: str
    """Filter by exact TADIG."""


class Page(TypedDict, total=False):
    number: int
    """The page number to load."""

    size: int
    """The size of the page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["EmbeddingResponse", "Data"]


class Data(BaseModel):
    created_at: Optional[str] = None

    finished_at: Optional[str] = None

    status: Optional[str] = None

    task_id: Optional[str] = None

    task_name: Optional[str] = None

    user_id: Optional[str] = None


class EmbeddingResponse(BaseModel):
    data: Data



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    OAuthTokenResponse,
    OAuthGrantsResponse,
    OAuthRegisterResponse,
    OAuthRetrieveResponse,
    OAuthIntrospectResponse,
    OAuthRetrieveJwksResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestOAuth:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        oauth = client.oauth.retrieve(
            "consent_token",
        )
        assert_matches_type(OAuthRetrieveResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.oauth.with_raw_response.retrieve(
            "consent_token",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth = response.parse()
        assert_matches_type(OAuthRetrieveResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.oauth.with_streaming_response.retrieve(
            "consent_token",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth = response.parse()
            assert_matches_type(OAuthRetrieveResponse, oauth, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `consent_token` but received ''"):
            client.oauth.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_grants(self, client: Telnyx) -> None:
        oauth = client.oauth.grants(
            allowed=True,
            consent_token="consent_token",
        )
        assert_matches_type(OAuthGrantsResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_grants(self, client: Telnyx) -> None:
        response = client.oauth.with_raw_response.grants(
            allowed=True,
            consent_token="consent_token",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth = response.parse()
        assert_matches_type(OAuthGrantsResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_grants(self, client: Telnyx) -> None:
        with client.oauth.with_streaming_response.grants(
            allowed=True,
            consent_token="consent_token",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth = response.parse()
            assert_matches_type(OAuthGrantsResponse, oauth, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_introspect(self, client: Telnyx) -> None:
        oauth = client.oauth.introspect(
            token="token",
        )
        assert_matches_type(OAuthIntrospectResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_introspect(self, client: Telnyx) -> None:
        response = client.oauth.with_raw_response.introspect(
            token="token",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth = response.parse()
        assert_matches_type(OAuthIntrospectResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_introspect(self, client: Telnyx) -> None:
        with client.oauth.with_streaming_response.introspect(
            token="token",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth = response.parse()
            assert_matches_type(OAuthIntrospectResponse, oauth, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_register(self, client: Telnyx) -> None:
        oauth = client.oauth.register()
        assert_matches_type(OAuthRegisterResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_register_with_all_params(self, client: Telnyx) -> None:
        oauth = client.oauth.register(
            client_name="My OAuth Application",
            grant_types=["authorization_code"],
            logo_uri="https://example.com",
            policy_uri="https://example.com",
            redirect_uris=["https://example.com/callback"],
            response_types=["string"],
            scope="admin",
            token_endpoint_auth_method="none",
            tos_uri="https://example.com",
        )
        assert_matches_type(OAuthRegisterResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_register(self, client: Telnyx) -> None:
        response = client.oauth.with_raw_response.register()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth = response.parse()
        assert_matches_type(OAuthRegisterResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_register(self, client: Telnyx) -> None:
        with client.oauth.with_streaming_response.register() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth = response.parse()
            assert_matches_type(OAuthRegisterResponse, oauth, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism doesn't properly handle redirects")
    @parametrize
    def test_method_retrieve_authorize(self, client: Telnyx) -> None:
        oauth = client.oauth.retrieve_authorize(
            client_id="client_id",
            redirect_uri="https://example.com",
            response_type="code",
        )
        assert oauth is None

    @pytest.mark.skip(reason="Prism doesn't properly handle redirects")
    @parametrize
    def test_method_retrieve_authorize_with_all_params(self, client: Telnyx) -> None:
        oauth = client.oauth.retrieve_authorize(
            client_id="client_id",
            redirect_uri="https://example.com",
            response_type="code",
            code_challenge="code_challenge",
            code_challenge_method="plain",
            scope="scope",
            state="state",
        )
        assert oauth is None

    @pytest.mark.skip(reason="Prism doesn't properly handle redirects")
    @parametrize
    def test_raw_response_retrieve_authorize(self, client: Telnyx) -> None:
        response = client.oauth.with_raw_response.retrieve_authorize(
            client_id="client_id",
            redirect_uri="https://example.com",
            response_type="code",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth = response.parse()
        assert oauth is None

    @pytest.mark.skip(reason="Prism doesn't properly handle redirects")
    @parametrize
    def test_streaming_response_retrieve_authorize(self, client: Telnyx) -> None:
        with client.oauth.with_streaming_response.retrieve_authorize(
            client_id="client_id",
            redirect_uri="https://example.com",
            response_type="code",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth = response.parse()
            assert oauth is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_jwks(self, client: Telnyx) -> None:
        oauth = client.oauth.retrieve_jwks()
        assert_matches_type(OAuthRetrieveJwksResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve_jwks(self, client: Telnyx) -> None:
        response = client.oauth.with_raw_response.retrieve_jwks()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth = response.parse()
        assert_matches_type(OAuthRetrieveJwksResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve_jwks(self, client: Telnyx) -> None:
        with client.oauth.with_streaming_response.retrieve_jwks() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth = response.parse()
            assert_matches_type(OAuthRetrieveJwksResponse, oauth, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_token(self, client: Telnyx) -> None:
        oauth = client.oauth.token(
            grant_type="client_credentials",
        )
        assert_matches_type(OAuthTokenResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_token_with_all_params(self, client: Telnyx) -> None:
        oauth = client.oauth.token(
            grant_type="client_credentials",
            client_id="client_id",
            client_secret="client_secret",
            code="code",
            code_verifier="code_verifier",
            redirect_uri="https://example.com",
            refresh_token="refresh_token",
            scope="admin",
        )
        assert_matches_type(OAuthTokenResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_token(self, client: Telnyx) -> None:
        response = client.oauth.with_raw_response.token(
            grant_type="client_credentials",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth = response.parse()
        assert_matches_type(OAuthTokenResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_token(self, client: Telnyx) -> None:
        with client.oauth.with_streaming_response.token(
            grant_type="client_credentials",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth = response.parse()
            assert_matches_type(OAuthTokenResponse, oauth, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncOAuth:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        oauth = await async_client.oauth.retrieve(
            "consent_token",
        )
        assert_matches_type(OAuthRetrieveResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.oauth.with_raw_response.retrieve(
            "consent_token",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth = await response.parse()
        assert_matches_type(OAuthRetrieveResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.oauth.with_streaming_response.retrieve(
            "consent_token",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth = await response.parse()
            assert_matches_type(OAuthRetrieveResponse, oauth, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `consent_token` but received ''"):
            await async_client.oauth.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_grants(self, async_client: AsyncTelnyx) -> None:
        oauth = await async_client.oauth.grants(
            allowed=True,
            consent_token="consent_token",
        )
        assert_matches_type(OAuthGrantsResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_grants(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.oauth.with_raw_response.grants(
            allowed=True,
            consent_token="consent_token",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth = await response.parse()
        assert_matches_type(OAuthGrantsResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_grants(self, async_client: AsyncTelnyx) -> None:
        async with async_client.oauth.with_streaming_response.grants(
            allowed=True,
            consent_token="consent_token",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth = await response.parse()
            assert_matches_type(OAuthGrantsResponse, oauth, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_introspect(self, async_client: AsyncTelnyx) -> None:
        oauth = await async_client.oauth.introspect(
            token="token",
        )
        assert_matches_type(OAuthIntrospectResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_introspect(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.oauth.with_raw_response.introspect(
            token="token",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth = await response.parse()
        assert_matches_type(OAuthIntrospectResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_introspect(self, async_client: AsyncTelnyx) -> None:
        async with async_client.oauth.with_streaming_response.introspect(
            token="token",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth = await response.parse()
            assert_matches_type(OAuthIntrospectResponse, oauth, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_register(self, async_client: AsyncTelnyx) -> None:
        oauth = await async_client.oauth.register()
        assert_matches_type(OAuthRegisterResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_register_with_all_params(self, async_client: AsyncTelnyx) -> None:
        oauth = await async_client.oauth.register(
            client_name="My OAuth Application",
            grant_types=["authorization_code"],
            logo_uri="https://example.com",
            policy_uri="https://example.com",
            redirect_uris=["https://example.com/callback"],
            response_types=["string"],
            scope="admin",
            token_endpoint_auth_method="none",
            tos_uri="https://example.com",
        )
        assert_matches_type(OAuthRegisterResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_register(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.oauth.with_raw_response.register()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth = await response.parse()
        assert_matches_type(OAuthRegisterResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_register(self, async_client: AsyncTelnyx) -> None:
        async with async_client.oauth.with_streaming_response.register() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth = await response.parse()
            assert_matches_type(OAuthRegisterResponse, oauth, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism doesn't properly handle redirects")
    @parametrize
    async def test_method_retrieve_authorize(self, async_client: AsyncTelnyx) -> None:
        oauth = await async_client.oauth.retrieve_authorize(
            client_id="client_id",
            redirect_uri="https://example.com",
            response_type="code",
        )
        assert oauth is None

    @pytest.mark.skip(reason="Prism doesn't properly handle redirects")
    @parametrize
    async def test_method_retrieve_authorize_with_all_params(self, async_client: AsyncTelnyx) -> None:
        oauth = await async_client.oauth.retrieve_authorize(
            client_id="client_id",
            redirect_uri="https://example.com",
            response_type="code",
            code_challenge="code_challenge",
            code_challenge_method="plain",
            scope="scope",
            state="state",
        )
        assert oauth is None

    @pytest.mark.skip(reason="Prism doesn't properly handle redirects")
    @parametrize
    async def test_raw_response_retrieve_authorize(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.oauth.with_raw_response.retrieve_authorize(
            client_id="client_id",
            redirect_uri="https://example.com",
            response_type="code",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth = await response.parse()
        assert oauth is None

    @pytest.mark.skip(reason="Prism doesn't properly handle redirects")
    @parametrize
    async def test_streaming_response_retrieve_authorize(self, async_client: AsyncTelnyx) -> None:
        async with async_client.oauth.with_streaming_response.retrieve_authorize(
            client_id="client_id",
            redirect_uri="https://example.com",
            response_type="code",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth = await response.parse()
            assert oauth is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_jwks(self, async_client: AsyncTelnyx) -> None:
        oauth = await async_client.oauth.retrieve_jwks()
        assert_matches_type(OAuthRetrieveJwksResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve_jwks(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.oauth.with_raw_response.retrieve_jwks()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth = await response.parse()
        assert_matches_type(OAuthRetrieveJwksResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve_jwks(self, async_client: AsyncTelnyx) -> None:
        async with async_client.oauth.with_streaming_response.retrieve_jwks() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth = await response.parse()
            assert_matches_type(OAuthRetrieveJwksResponse, oauth, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_token(self, async_client: AsyncTelnyx) -> None:
        oauth = await async_client.oauth.token(
            grant_type="client_credentials",
        )
        assert_matches_type(OAuthTokenResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_token_with_all_params(self, async_client: AsyncTelnyx) -> None:
        oauth = await async_client.oauth.token(
            grant_type="client_credentials",
            client_id="client_id",
            client_secret="client_secret",
            code="code",
            code_verifier="code_verifier",
            redirect_uri="https://example.com",
            refresh_token="refresh_token",
            scope="admin",
        )
        assert_matches_type(OAuthTokenResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_token(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.oauth.with_raw_response.token(
            grant_type="client_credentials",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        oauth = await response.parse()
        assert_matches_type(OAuthTokenResponse, oauth, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_token(self, async_client: AsyncTelnyx) -> None:
        async with async_client.oauth.with_streaming_response.token(
            grant_type="client_credentials",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            oauth = await response.parse()
            assert_matches_type(OAuthTokenResponse, oauth, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["CustomerServiceRecordStatusChangedWebhookEvent", "Data", "DataPayload", "Meta"]


class DataPayload(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the customer service record."""

    phone_number: Optional[str] = None
    """The phone number of the customer service record."""

    status: Optional[Literal["pending", "completed", "failed"]] = None
    """The status of the customer service record"""

    updated_at: Optional[datetime] = None
    """
    ISO 8601 formatted date indicating the last time where the resource was updated.
    """


class Data(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the callback event."""

    event_type: Optional[Literal["customer_service_record.status_changed"]] = None
    """The type of the callback event."""

    occurred_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the callback event occurred."""

    payload: Optional[DataPayload] = None

    record_type: Optional[Literal["event"]] = None
    """Identifies the type of the resource."""


class Meta(BaseModel):
    attempt: Optional[int] = None
    """The number of times the callback webhook has been attempted."""

    delivered_to: Optional[str] = None
    """The URL that the callback webhook was delivered to."""


class CustomerServiceRecordStatusChangedWebhookEvent(BaseModel):
    data: Optional[Data] = None

    meta: Optional[Meta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["PortingOrderDocumentsParam"]


class PortingOrderDocumentsParam(TypedDict, total=False):
    invoice: str
    """Returned ID of the submitted Invoice via the Documents endpoint"""

    loa: str
    """Returned ID of the submitted LOA via the Documents endpoint"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .texml_application import TexmlApplication

__all__ = ["TexmlApplicationListResponse", "Meta"]


class Meta(BaseModel):
    page_number: Optional[int] = None

    page_size: Optional[int] = None

    total_pages: Optional[int] = None

    total_results: Optional[int] = None


class TexmlApplicationListResponse(BaseModel):
    data: Optional[List[TexmlApplication]] = None

    meta: Optional[Meta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import messaging_numbers_bulk_update_create_params
from .._types import Body, Query, Headers, NotGiven, SequenceNotStr, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.messaging_numbers_bulk_update_create_response import MessagingNumbersBulkUpdateCreateResponse
from ..types.messaging_numbers_bulk_update_retrieve_response import MessagingNumbersBulkUpdateRetrieveResponse

__all__ = ["MessagingNumbersBulkUpdatesResource", "AsyncMessagingNumbersBulkUpdatesResource"]


class MessagingNumbersBulkUpdatesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MessagingNumbersBulkUpdatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return MessagingNumbersBulkUpdatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MessagingNumbersBulkUpdatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return MessagingNumbersBulkUpdatesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        messaging_profile_id: str,
        numbers: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingNumbersBulkUpdateCreateResponse:
        """
        Update the messaging profile of multiple phone numbers

        Args:
          messaging_profile_id:
              Configure the messaging profile these phone numbers are assigned to:

              - Set this field to `""` to unassign each number from their respective messaging
                profile
              - Set this field to a quoted UUID of a messaging profile to assign these numbers
                to that messaging profile

          numbers: The list of phone numbers to update.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/messaging_numbers_bulk_updates",
            body=maybe_transform(
                {
                    "messaging_profile_id": messaging_profile_id,
                    "numbers": numbers,
                },
                messaging_numbers_bulk_update_create_params.MessagingNumbersBulkUpdateCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingNumbersBulkUpdateCreateResponse,
        )

    def retrieve(
        self,
        order_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingNumbersBulkUpdateRetrieveResponse:
        """
        Retrieve bulk update status

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not order_id:
            raise ValueError(f"Expected a non-empty value for `order_id` but received {order_id!r}")
        return self._get(
            f"/messaging_numbers_bulk_updates/{order_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingNumbersBulkUpdateRetrieveResponse,
        )


class AsyncMessagingNumbersBulkUpdatesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMessagingNumbersBulkUpdatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMessagingNumbersBulkUpdatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMessagingNumbersBulkUpdatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncMessagingNumbersBulkUpdatesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        messaging_profile_id: str,
        numbers: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingNumbersBulkUpdateCreateResponse:
        """
        Update the messaging profile of multiple phone numbers

        Args:
          messaging_profile_id:
              Configure the messaging profile these phone numbers are assigned to:

              - Set this field to `""` to unassign each number from their respective messaging
                profile
              - Set this field to a quoted UUID of a messaging profile to assign these numbers
                to that messaging profile

          numbers: The list of phone numbers to update.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/messaging_numbers_bulk_updates",
            body=await async_maybe_transform(
                {
                    "messaging_profile_id": messaging_profile_id,
                    "numbers": numbers,
                },
                messaging_numbers_bulk_update_create_params.MessagingNumbersBulkUpdateCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingNumbersBulkUpdateCreateResponse,
        )

    async def retrieve(
        self,
        order_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessagingNumbersBulkUpdateRetrieveResponse:
        """
        Retrieve bulk update status

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not order_id:
            raise ValueError(f"Expected a non-empty value for `order_id` but received {order_id!r}")
        return await self._get(
            f"/messaging_numbers_bulk_updates/{order_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessagingNumbersBulkUpdateRetrieveResponse,
        )


class MessagingNumbersBulkUpdatesResourceWithRawResponse:
    def __init__(self, messaging_numbers_bulk_updates: MessagingNumbersBulkUpdatesResource) -> None:
        self._messaging_numbers_bulk_updates = messaging_numbers_bulk_updates

        self.create = to_raw_response_wrapper(
            messaging_numbers_bulk_updates.create,
        )
        self.retrieve = to_raw_response_wrapper(
            messaging_numbers_bulk_updates.retrieve,
        )


class AsyncMessagingNumbersBulkUpdatesResourceWithRawResponse:
    def __init__(self, messaging_numbers_bulk_updates: AsyncMessagingNumbersBulkUpdatesResource) -> None:
        self._messaging_numbers_bulk_updates = messaging_numbers_bulk_updates

        self.create = async_to_raw_response_wrapper(
            messaging_numbers_bulk_updates.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            messaging_numbers_bulk_updates.retrieve,
        )


class MessagingNumbersBulkUpdatesResourceWithStreamingResponse:
    def __init__(self, messaging_numbers_bulk_updates: MessagingNumbersBulkUpdatesResource) -> None:
        self._messaging_numbers_bulk_updates = messaging_numbers_bulk_updates

        self.create = to_streamed_response_wrapper(
            messaging_numbers_bulk_updates.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            messaging_numbers_bulk_updates.retrieve,
        )


class AsyncMessagingNumbersBulkUpdatesResourceWithStreamingResponse:
    def __init__(self, messaging_numbers_bulk_updates: AsyncMessagingNumbersBulkUpdatesResource) -> None:
        self._messaging_numbers_bulk_updates = messaging_numbers_bulk_updates

        self.create = async_to_streamed_response_wrapper(
            messaging_numbers_bulk_updates.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            messaging_numbers_bulk_updates.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["AuditEventListParams", "Filter", "Page"]


class AuditEventListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[created_before], filter[created_after]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[number], page[size]
    """

    sort: Literal["asc", "desc"]
    """Set the order of the results by the creation date."""


class Filter(TypedDict, total=False):
    created_after: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter for audit events created after a specific date."""

    created_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter for audit events created before a specific date."""


class Page(TypedDict, total=False):
    number: int
    """Page number to load."""

    size: int
    """Number of items per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["ActionJoinParams"]


class ActionJoinParams(TypedDict, total=False):
    call_control_id: Required[str]
    """Unique identifier and token for controlling the call"""

    beep_enabled: Literal["always", "never", "on_enter", "on_exit"]
    """
    Whether a beep sound should be played when the participant joins and/or leaves
    the conference. Can be used to override the conference-level setting.
    """

    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string. Please note that the client_state
    will be updated for the participient call leg and the change will not affect
    conferencing webhooks unless the participient is the owner of the conference.
    """

    command_id: str
    """Use this field to avoid execution of duplicate commands.

    Telnyx will ignore subsequent commands with the same `command_id` as one that
    has already been executed.
    """

    end_conference_on_exit: bool
    """
    Whether the conference should end and all remaining participants be hung up
    after the participant leaves the conference. Defaults to "false".
    """

    hold: bool
    """
    Whether the participant should be put on hold immediately after joining the
    conference. Defaults to "false".
    """

    hold_audio_url: str
    """
    The URL of a file to be played to the participant when they are put on hold
    after joining the conference. hold_media_name and hold_audio_url cannot be used
    together in one request. Takes effect only when "start_conference_on_create" is
    set to "false". This property takes effect only if "hold" is set to "true".
    """

    hold_media_name: str
    """
    The media_name of a file to be played to the participant when they are put on
    hold after joining the conference. The media_name must point to a file
    previously uploaded to api.telnyx.com/v2/media by the same user/organization.
    The file must either be a WAV or MP3 file. Takes effect only when
    "start_conference_on_create" is set to "false". This property takes effect only
    if "hold" is set to "true".
    """

    mute: bool
    """Whether the participant should be muted immediately after joining the
    conference.

    Defaults to "false".
    """

    soft_end_conference_on_exit: bool
    """Whether the conference should end after the participant leaves the conference.

    NOTE this doesn't hang up the other participants. Defaults to "false".
    """

    start_conference_on_enter: bool
    """
    Whether the conference should be started after the participant joins the
    conference. Defaults to "false".
    """

    supervisor_role: Literal["barge", "monitor", "none", "whisper"]
    """Sets the joining participant as a supervisor for the conference.

    A conference can have multiple supervisors. "barge" means the supervisor enters
    the conference as a normal participant. This is the same as "none". "monitor"
    means the supervisor is muted but can hear all participants. "whisper" means
    that only the specified "whisper_call_control_ids" can hear the supervisor.
    Defaults to "none".
    """

    whisper_call_control_ids: SequenceNotStr[str]
    """Array of unique call_control_ids the joining supervisor can whisper to.

    If none provided, the supervisor will join the conference as a monitoring
    participant only.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .voice import (
    VoiceResource,
    AsyncVoiceResource,
    VoiceResourceWithRawResponse,
    AsyncVoiceResourceWithRawResponse,
    VoiceResourceWithStreamingResponse,
    AsyncVoiceResourceWithStreamingResponse,
)
from .messaging import (
    MessagingResource,
    AsyncMessagingResource,
    MessagingResourceWithRawResponse,
    AsyncMessagingResourceWithRawResponse,
    MessagingResourceWithStreamingResponse,
    AsyncMessagingResourceWithStreamingResponse,
)
from .number_lookup import (
    NumberLookupResource,
    AsyncNumberLookupResource,
    NumberLookupResourceWithRawResponse,
    AsyncNumberLookupResourceWithRawResponse,
    NumberLookupResourceWithStreamingResponse,
    AsyncNumberLookupResourceWithStreamingResponse,
)
from .usage_reports import (
    UsageReportsResource,
    AsyncUsageReportsResource,
    UsageReportsResourceWithRawResponse,
    AsyncUsageReportsResourceWithRawResponse,
    UsageReportsResourceWithStreamingResponse,
    AsyncUsageReportsResourceWithStreamingResponse,
)

__all__ = [
    "MessagingResource",
    "AsyncMessagingResource",
    "MessagingResourceWithRawResponse",
    "AsyncMessagingResourceWithRawResponse",
    "MessagingResourceWithStreamingResponse",
    "AsyncMessagingResourceWithStreamingResponse",
    "NumberLookupResource",
    "AsyncNumberLookupResource",
    "NumberLookupResourceWithRawResponse",
    "AsyncNumberLookupResourceWithRawResponse",
    "NumberLookupResourceWithStreamingResponse",
    "AsyncNumberLookupResourceWithStreamingResponse",
    "VoiceResource",
    "AsyncVoiceResource",
    "VoiceResourceWithRawResponse",
    "AsyncVoiceResourceWithRawResponse",
    "VoiceResourceWithStreamingResponse",
    "AsyncVoiceResourceWithStreamingResponse",
    "UsageReportsResource",
    "AsyncUsageReportsResource",
    "UsageReportsResourceWithRawResponse",
    "AsyncUsageReportsResourceWithRawResponse",
    "UsageReportsResourceWithStreamingResponse",
    "AsyncUsageReportsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["CallConversationEndedWebhookEvent", "Data", "DataPayload"]


class DataPayload(BaseModel):
    assistant_id: Optional[str] = None
    """Unique identifier of the assistant involved in the call."""

    call_control_id: Optional[str] = None
    """Call ID used to issue commands via Call Control API."""

    call_leg_id: Optional[str] = None
    """ID that is unique to the call leg."""

    call_session_id: Optional[str] = None
    """ID that is unique to the call session (group of related call legs)."""

    calling_party_type: Optional[Literal["pstn", "sip"]] = None
    """The type of calling party connection."""

    client_state: Optional[str] = None
    """Base64-encoded state received from a command."""

    connection_id: Optional[str] = None
    """Call Control App ID (formerly Telnyx connection ID) used in the call."""

    conversation_id: Optional[str] = None
    """ID unique to the conversation or insight group generated for the call."""

    duration_sec: Optional[int] = None
    """Duration of the conversation in seconds."""

    from_: Optional[str] = FieldInfo(alias="from", default=None)
    """The caller's number or identifier."""

    llm_model: Optional[str] = None
    """The large language model used during the conversation."""

    stt_model: Optional[str] = None
    """The speech-to-text model used in the conversation."""

    to: Optional[str] = None
    """The callee's number or SIP address."""

    tts_model_id: Optional[str] = None
    """The model ID used for text-to-speech synthesis."""

    tts_provider: Optional[str] = None
    """The text-to-speech provider used in the call."""

    tts_voice_id: Optional[str] = None
    """Voice ID used for TTS."""


class Data(BaseModel):
    id: Optional[str] = None
    """Unique identifier for the event."""

    created_at: Optional[datetime] = None
    """Timestamp when the event was created in the system."""

    event_type: Optional[Literal["call.conversation.ended"]] = None
    """The type of event being delivered."""

    occurred_at: Optional[datetime] = None
    """ISO 8601 datetime of when the event occurred."""

    payload: Optional[DataPayload] = None

    record_type: Optional[Literal["event"]] = None
    """Identifies the type of the resource."""


class CallConversationEndedWebhookEvent(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from ..external_voice_integrations_pagination_meta import ExternalVoiceIntegrationsPaginationMeta

__all__ = ["LogMessageListResponse", "LogMessage", "LogMessageMeta", "LogMessageSource"]


class LogMessageMeta(BaseModel):
    external_connection_id: Optional[str] = None
    """The external connection the log message is associated with, if any."""

    telephone_number: Optional[str] = None
    """The telephone number the log message is associated with, if any."""

    ticket_id: Optional[str] = None
    """The ticket ID for an operation that generated the log message, if any."""


class LogMessageSource(BaseModel):
    pointer: Optional[str] = None
    """JSON pointer (RFC6901) to the offending entity."""


class LogMessage(BaseModel):
    code: str

    title: str

    detail: Optional[str] = None

    meta: Optional[LogMessageMeta] = None

    source: Optional[LogMessageSource] = None


class LogMessageListResponse(BaseModel):
    log_messages: Optional[List[LogMessage]] = None

    meta: Optional[ExternalVoiceIntegrationsPaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import TypedDict

__all__ = ["GlobalIPHealthCheckCreateParams"]


class GlobalIPHealthCheckCreateParams(TypedDict, total=False):
    global_ip_id: str
    """Global IP ID."""

    health_check_params: Dict[str, object]
    """A Global IP health check params."""

    health_check_type: str
    """The Global IP health check type."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["PhoneNumberDetailed"]


class PhoneNumberDetailed(BaseModel):
    id: Optional[str] = None
    """Identifies the resource."""

    billing_group_id: Optional[str] = None
    """Identifies the billing group associated with the phone number."""

    call_forwarding_enabled: Optional[bool] = None
    """
    Indicates if call forwarding will be enabled for this number if forwards_to and
    forwarding_type are filled in. Defaults to true for backwards compatibility with
    APIV1 use of numbers endpoints.
    """

    call_recording_enabled: Optional[bool] = None
    """Indicates whether call recording is enabled for this number."""

    caller_id_name_enabled: Optional[bool] = None
    """Indicates whether caller ID is enabled for this number."""

    cnam_listing_enabled: Optional[bool] = None
    """Indicates whether a CNAM listing is enabled for this number."""

    connection_id: Optional[str] = None
    """Identifies the connection associated with the phone number."""

    connection_name: Optional[str] = None
    """
    The user-assigned name of the connection to be associated with this phone
    number.
    """

    country_iso_alpha2: Optional[str] = None
    """The ISO 3166-1 alpha-2 country code of the phone number."""

    created_at: Optional[str] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    customer_reference: Optional[str] = None
    """A customer reference string for customer look ups."""

    deletion_lock_enabled: Optional[bool] = None
    """Indicates whether deletion lock is enabled for this number.

    When enabled, this prevents the phone number from being deleted via the API or
    Telnyx portal.
    """

    emergency_address_id: Optional[str] = None
    """Identifies the emergency address associated with the phone number."""

    emergency_enabled: Optional[bool] = None
    """Indicates whether emergency services are enabled for this number."""

    emergency_status: Optional[
        Literal["active", "deprovisioning", "disabled", "provisioning", "provisioning-failed"]
    ] = None
    """
    Indicates the status of the provisioning of emergency services for the phone
    number. This field contains information about activity that may be ongoing for a
    number where it either is being provisioned or deprovisioned but is not yet
    enabled/disabled.
    """

    external_pin: Optional[str] = None
    """
    If someone attempts to port your phone number away from Telnyx and your phone
    number has an external PIN set, Telnyx will attempt to verify that you provided
    the correct external PIN to the winning carrier. Note that not all carriers
    cooperate with this security mechanism.
    """

    inbound_call_screening: Optional[Literal["disabled", "reject_calls", "flag_calls"]] = None
    """
    The inbound_call_screening setting is a phone number configuration option
    variable that allows users to configure their settings to block or flag
    fraudulent calls. It can be set to disabled, reject_calls, or flag_calls. This
    feature has an additional per-number monthly cost associated with it.
    """

    messaging_profile_id: Optional[str] = None
    """Identifies the messaging profile associated with the phone number."""

    messaging_profile_name: Optional[str] = None
    """The name of the messaging profile associated with the phone number."""

    phone_number: Optional[str] = None
    """The +E.164-formatted phone number associated with this record."""

    phone_number_type: Optional[
        Literal[
            "local", "toll_free", "mobile", "national", "shared_cost", "landline", "tollfree", "shortcode", "longcode"
        ]
    ] = None
    """
    The phone number's type. Note: For numbers purchased prior to July 2023 or when
    fetching a number's details immediately after a purchase completes, the legacy
    values `tollfree`, `shortcode` or `longcode` may be returned instead.
    """

    purchased_at: Optional[str] = None
    """ISO 8601 formatted date indicating when the resource was purchased."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    source_type: Optional[Literal["number_order", "port_request"]] = None
    """Indicates if the phone number was purchased or ported in.

    For some numbers this information may not be available.
    """

    status: Optional[
        Literal[
            "purchase-pending",
            "purchase-failed",
            "port-pending",
            "port-failed",
            "active",
            "deleted",
            "emergency-only",
            "ported-out",
            "port-out-pending",
            "requirement-info-pending",
            "requirement-info-under-review",
            "requirement-info-exception",
            "provision-pending",
        ]
    ] = None
    """The phone number's current status."""

    t38_fax_gateway_enabled: Optional[bool] = None
    """Indicates whether T38 Fax Gateway for inbound calls to this number."""

    tags: Optional[List[str]] = None
    """A list of user-assigned tags to help manage the phone number."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import invoice_list_params, invoice_retrieve_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.invoice_list_response import InvoiceListResponse
from ..types.invoice_retrieve_response import InvoiceRetrieveResponse

__all__ = ["InvoicesResource", "AsyncInvoicesResource"]


class InvoicesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> InvoicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return InvoicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InvoicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return InvoicesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        action: Literal["json", "link"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InvoiceRetrieveResponse:
        """
        Retrieve a single invoice by its unique identifier.

        Args:
          action: Invoice action

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/invoices/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"action": action}, invoice_retrieve_params.InvoiceRetrieveParams),
            ),
            cast_to=InvoiceRetrieveResponse,
        )

    def list(
        self,
        *,
        page: invoice_list_params.Page | Omit = omit,
        sort: Literal["period_start", "-period_start"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InvoiceListResponse:
        """
        Retrieve a paginated list of invoices.

        Args:
          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          sort: Specifies the sort order for results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/invoices",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "sort": sort,
                    },
                    invoice_list_params.InvoiceListParams,
                ),
            ),
            cast_to=InvoiceListResponse,
        )


class AsyncInvoicesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncInvoicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInvoicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInvoicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncInvoicesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        action: Literal["json", "link"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InvoiceRetrieveResponse:
        """
        Retrieve a single invoice by its unique identifier.

        Args:
          action: Invoice action

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/invoices/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"action": action}, invoice_retrieve_params.InvoiceRetrieveParams),
            ),
            cast_to=InvoiceRetrieveResponse,
        )

    async def list(
        self,
        *,
        page: invoice_list_params.Page | Omit = omit,
        sort: Literal["period_start", "-period_start"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InvoiceListResponse:
        """
        Retrieve a paginated list of invoices.

        Args:
          page: Consolidated page parameter (deepObject style). Originally: page[number],
              page[size]

          sort: Specifies the sort order for results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/invoices",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "page": page,
                        "sort": sort,
                    },
                    invoice_list_params.InvoiceListParams,
                ),
            ),
            cast_to=InvoiceListResponse,
        )


class InvoicesResourceWithRawResponse:
    def __init__(self, invoices: InvoicesResource) -> None:
        self._invoices = invoices

        self.retrieve = to_raw_response_wrapper(
            invoices.retrieve,
        )
        self.list = to_raw_response_wrapper(
            invoices.list,
        )


class AsyncInvoicesResourceWithRawResponse:
    def __init__(self, invoices: AsyncInvoicesResource) -> None:
        self._invoices = invoices

        self.retrieve = async_to_raw_response_wrapper(
            invoices.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            invoices.list,
        )


class InvoicesResourceWithStreamingResponse:
    def __init__(self, invoices: InvoicesResource) -> None:
        self._invoices = invoices

        self.retrieve = to_streamed_response_wrapper(
            invoices.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            invoices.list,
        )


class AsyncInvoicesResourceWithStreamingResponse:
    def __init__(self, invoices: AsyncInvoicesResource) -> None:
        self._invoices = invoices

        self.retrieve = async_to_streamed_response_wrapper(
            invoices.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            invoices.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .event_status import EventStatus as EventStatus
from .assistant_test import AssistantTest as AssistantTest
from .version_config import VersionConfig as VersionConfig
from .test_list_params import TestListParams as TestListParams
from .tool_test_params import ToolTestParams as ToolTestParams
from .test_create_params import TestCreateParams as TestCreateParams
from .test_list_response import TestListResponse as TestListResponse
from .test_update_params import TestUpdateParams as TestUpdateParams
from .tool_test_response import ToolTestResponse as ToolTestResponse
from .version_config_param import VersionConfigParam as VersionConfigParam
from .version_update_params import VersionUpdateParams as VersionUpdateParams
from .canary_deploy_response import CanaryDeployResponse as CanaryDeployResponse
from .version_retrieve_params import VersionRetrieveParams as VersionRetrieveParams
from .scheduled_event_response import ScheduledEventResponse as ScheduledEventResponse
from .conversation_channel_type import ConversationChannelType as ConversationChannelType
from .canary_deploy_create_params import CanaryDeployCreateParams as CanaryDeployCreateParams
from .canary_deploy_update_params import CanaryDeployUpdateParams as CanaryDeployUpdateParams
from .scheduled_event_list_params import ScheduledEventListParams as ScheduledEventListParams
from .telnyx_conversation_channel import TelnyxConversationChannel as TelnyxConversationChannel
from .scheduled_sms_event_response import ScheduledSMSEventResponse as ScheduledSMSEventResponse
from .scheduled_event_create_params import ScheduledEventCreateParams as ScheduledEventCreateParams
from .scheduled_event_list_response import ScheduledEventListResponse as ScheduledEventListResponse
from .scheduled_phone_call_event_response import ScheduledPhoneCallEventResponse as ScheduledPhoneCallEventResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..types import (
    oauth_token_params,
    oauth_grants_params,
    oauth_register_params,
    oauth_introspect_params,
    oauth_retrieve_authorize_params,
)
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.oauth_token_response import OAuthTokenResponse
from ..types.oauth_grants_response import OAuthGrantsResponse
from ..types.oauth_register_response import OAuthRegisterResponse
from ..types.oauth_retrieve_response import OAuthRetrieveResponse
from ..types.oauth_introspect_response import OAuthIntrospectResponse
from ..types.oauth_retrieve_jwks_response import OAuthRetrieveJwksResponse

__all__ = ["OAuthResource", "AsyncOAuthResource"]


class OAuthResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> OAuthResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return OAuthResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OAuthResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return OAuthResourceWithStreamingResponse(self)

    def retrieve(
        self,
        consent_token: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthRetrieveResponse:
        """
        Retrieve details about an OAuth consent token

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not consent_token:
            raise ValueError(f"Expected a non-empty value for `consent_token` but received {consent_token!r}")
        return self._get(
            f"/oauth/consent/{consent_token}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthRetrieveResponse,
        )

    def grants(
        self,
        *,
        allowed: bool,
        consent_token: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthGrantsResponse:
        """
        Create an OAuth authorization grant

        Args:
          allowed: Whether the grant is allowed

          consent_token: Consent token

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/oauth/grants",
            body=maybe_transform(
                {
                    "allowed": allowed,
                    "consent_token": consent_token,
                },
                oauth_grants_params.OAuthGrantsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthGrantsResponse,
        )

    def introspect(
        self,
        *,
        token: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthIntrospectResponse:
        """
        Introspect an OAuth access token to check its validity and metadata

        Args:
          token: The token to introspect

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/oauth/introspect",
            body=maybe_transform({"token": token}, oauth_introspect_params.OAuthIntrospectParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthIntrospectResponse,
        )

    def register(
        self,
        *,
        client_name: str | Omit = omit,
        grant_types: List[Literal["authorization_code", "client_credentials", "refresh_token"]] | Omit = omit,
        logo_uri: str | Omit = omit,
        policy_uri: str | Omit = omit,
        redirect_uris: SequenceNotStr[str] | Omit = omit,
        response_types: SequenceNotStr[str] | Omit = omit,
        scope: str | Omit = omit,
        token_endpoint_auth_method: Literal["none", "client_secret_basic", "client_secret_post"] | Omit = omit,
        tos_uri: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthRegisterResponse:
        """
        Register a new OAuth client dynamically (RFC 7591)

        Args:
          client_name: Human-readable string name of the client to be presented to the end-user

          grant_types: Array of OAuth 2.0 grant type strings that the client may use

          logo_uri: URL of the client logo

          policy_uri: URL of the client's privacy policy

          redirect_uris: Array of redirection URI strings for use in redirect-based flows

          response_types: Array of the OAuth 2.0 response type strings that the client may use

          scope: Space-separated string of scope values that the client may use

          token_endpoint_auth_method: Authentication method for the token endpoint

          tos_uri: URL of the client's terms of service

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/oauth/register",
            body=maybe_transform(
                {
                    "client_name": client_name,
                    "grant_types": grant_types,
                    "logo_uri": logo_uri,
                    "policy_uri": policy_uri,
                    "redirect_uris": redirect_uris,
                    "response_types": response_types,
                    "scope": scope,
                    "token_endpoint_auth_method": token_endpoint_auth_method,
                    "tos_uri": tos_uri,
                },
                oauth_register_params.OAuthRegisterParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthRegisterResponse,
        )

    def retrieve_authorize(
        self,
        *,
        client_id: str,
        redirect_uri: str,
        response_type: Literal["code"],
        code_challenge: str | Omit = omit,
        code_challenge_method: Literal["plain", "S256"] | Omit = omit,
        scope: str | Omit = omit,
        state: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        OAuth 2.0 authorization endpoint for the authorization code flow

        Args:
          client_id: OAuth client identifier

          redirect_uri: Redirect URI

          response_type: OAuth response type

          code_challenge: PKCE code challenge

          code_challenge_method: PKCE code challenge method

          scope: Space-separated list of requested scopes

          state: State parameter for CSRF protection

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._get(
            "/oauth/authorize",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "client_id": client_id,
                        "redirect_uri": redirect_uri,
                        "response_type": response_type,
                        "code_challenge": code_challenge,
                        "code_challenge_method": code_challenge_method,
                        "scope": scope,
                        "state": state,
                    },
                    oauth_retrieve_authorize_params.OAuthRetrieveAuthorizeParams,
                ),
            ),
            cast_to=NoneType,
        )

    def retrieve_jwks(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthRetrieveJwksResponse:
        """Retrieve the JSON Web Key Set for token verification"""
        return self._get(
            "/oauth/jwks",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthRetrieveJwksResponse,
        )

    def token(
        self,
        *,
        grant_type: Literal["client_credentials", "authorization_code", "refresh_token"],
        client_id: str | Omit = omit,
        client_secret: str | Omit = omit,
        code: str | Omit = omit,
        code_verifier: str | Omit = omit,
        redirect_uri: str | Omit = omit,
        refresh_token: str | Omit = omit,
        scope: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthTokenResponse:
        """
        Exchange authorization code, client credentials, or refresh token for access
        token

        Args:
          grant_type: OAuth 2.0 grant type

          client_id: OAuth client ID (if not using HTTP Basic auth)

          client_secret: OAuth client secret (if not using HTTP Basic auth)

          code: Authorization code (for authorization_code flow)

          code_verifier: PKCE code verifier (for authorization_code flow)

          redirect_uri: Redirect URI (for authorization_code flow)

          refresh_token: Refresh token (for refresh_token flow)

          scope: Space-separated list of requested scopes (for client_credentials)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/oauth/token",
            body=maybe_transform(
                {
                    "grant_type": grant_type,
                    "client_id": client_id,
                    "client_secret": client_secret,
                    "code": code,
                    "code_verifier": code_verifier,
                    "redirect_uri": redirect_uri,
                    "refresh_token": refresh_token,
                    "scope": scope,
                },
                oauth_token_params.OAuthTokenParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthTokenResponse,
        )


class AsyncOAuthResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncOAuthResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOAuthResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOAuthResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncOAuthResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        consent_token: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthRetrieveResponse:
        """
        Retrieve details about an OAuth consent token

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not consent_token:
            raise ValueError(f"Expected a non-empty value for `consent_token` but received {consent_token!r}")
        return await self._get(
            f"/oauth/consent/{consent_token}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthRetrieveResponse,
        )

    async def grants(
        self,
        *,
        allowed: bool,
        consent_token: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthGrantsResponse:
        """
        Create an OAuth authorization grant

        Args:
          allowed: Whether the grant is allowed

          consent_token: Consent token

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/oauth/grants",
            body=await async_maybe_transform(
                {
                    "allowed": allowed,
                    "consent_token": consent_token,
                },
                oauth_grants_params.OAuthGrantsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthGrantsResponse,
        )

    async def introspect(
        self,
        *,
        token: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthIntrospectResponse:
        """
        Introspect an OAuth access token to check its validity and metadata

        Args:
          token: The token to introspect

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/oauth/introspect",
            body=await async_maybe_transform({"token": token}, oauth_introspect_params.OAuthIntrospectParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthIntrospectResponse,
        )

    async def register(
        self,
        *,
        client_name: str | Omit = omit,
        grant_types: List[Literal["authorization_code", "client_credentials", "refresh_token"]] | Omit = omit,
        logo_uri: str | Omit = omit,
        policy_uri: str | Omit = omit,
        redirect_uris: SequenceNotStr[str] | Omit = omit,
        response_types: SequenceNotStr[str] | Omit = omit,
        scope: str | Omit = omit,
        token_endpoint_auth_method: Literal["none", "client_secret_basic", "client_secret_post"] | Omit = omit,
        tos_uri: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthRegisterResponse:
        """
        Register a new OAuth client dynamically (RFC 7591)

        Args:
          client_name: Human-readable string name of the client to be presented to the end-user

          grant_types: Array of OAuth 2.0 grant type strings that the client may use

          logo_uri: URL of the client logo

          policy_uri: URL of the client's privacy policy

          redirect_uris: Array of redirection URI strings for use in redirect-based flows

          response_types: Array of the OAuth 2.0 response type strings that the client may use

          scope: Space-separated string of scope values that the client may use

          token_endpoint_auth_method: Authentication method for the token endpoint

          tos_uri: URL of the client's terms of service

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/oauth/register",
            body=await async_maybe_transform(
                {
                    "client_name": client_name,
                    "grant_types": grant_types,
                    "logo_uri": logo_uri,
                    "policy_uri": policy_uri,
                    "redirect_uris": redirect_uris,
                    "response_types": response_types,
                    "scope": scope,
                    "token_endpoint_auth_method": token_endpoint_auth_method,
                    "tos_uri": tos_uri,
                },
                oauth_register_params.OAuthRegisterParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthRegisterResponse,
        )

    async def retrieve_authorize(
        self,
        *,
        client_id: str,
        redirect_uri: str,
        response_type: Literal["code"],
        code_challenge: str | Omit = omit,
        code_challenge_method: Literal["plain", "S256"] | Omit = omit,
        scope: str | Omit = omit,
        state: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        OAuth 2.0 authorization endpoint for the authorization code flow

        Args:
          client_id: OAuth client identifier

          redirect_uri: Redirect URI

          response_type: OAuth response type

          code_challenge: PKCE code challenge

          code_challenge_method: PKCE code challenge method

          scope: Space-separated list of requested scopes

          state: State parameter for CSRF protection

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._get(
            "/oauth/authorize",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "client_id": client_id,
                        "redirect_uri": redirect_uri,
                        "response_type": response_type,
                        "code_challenge": code_challenge,
                        "code_challenge_method": code_challenge_method,
                        "scope": scope,
                        "state": state,
                    },
                    oauth_retrieve_authorize_params.OAuthRetrieveAuthorizeParams,
                ),
            ),
            cast_to=NoneType,
        )

    async def retrieve_jwks(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthRetrieveJwksResponse:
        """Retrieve the JSON Web Key Set for token verification"""
        return await self._get(
            "/oauth/jwks",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthRetrieveJwksResponse,
        )

    async def token(
        self,
        *,
        grant_type: Literal["client_credentials", "authorization_code", "refresh_token"],
        client_id: str | Omit = omit,
        client_secret: str | Omit = omit,
        code: str | Omit = omit,
        code_verifier: str | Omit = omit,
        redirect_uri: str | Omit = omit,
        refresh_token: str | Omit = omit,
        scope: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OAuthTokenResponse:
        """
        Exchange authorization code, client credentials, or refresh token for access
        token

        Args:
          grant_type: OAuth 2.0 grant type

          client_id: OAuth client ID (if not using HTTP Basic auth)

          client_secret: OAuth client secret (if not using HTTP Basic auth)

          code: Authorization code (for authorization_code flow)

          code_verifier: PKCE code verifier (for authorization_code flow)

          redirect_uri: Redirect URI (for authorization_code flow)

          refresh_token: Refresh token (for refresh_token flow)

          scope: Space-separated list of requested scopes (for client_credentials)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/oauth/token",
            body=await async_maybe_transform(
                {
                    "grant_type": grant_type,
                    "client_id": client_id,
                    "client_secret": client_secret,
                    "code": code,
                    "code_verifier": code_verifier,
                    "redirect_uri": redirect_uri,
                    "refresh_token": refresh_token,
                    "scope": scope,
                },
                oauth_token_params.OAuthTokenParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OAuthTokenResponse,
        )


class OAuthResourceWithRawResponse:
    def __init__(self, oauth: OAuthResource) -> None:
        self._oauth = oauth

        self.retrieve = to_raw_response_wrapper(
            oauth.retrieve,
        )
        self.grants = to_raw_response_wrapper(
            oauth.grants,
        )
        self.introspect = to_raw_response_wrapper(
            oauth.introspect,
        )
        self.register = to_raw_response_wrapper(
            oauth.register,
        )
        self.retrieve_authorize = to_raw_response_wrapper(
            oauth.retrieve_authorize,
        )
        self.retrieve_jwks = to_raw_response_wrapper(
            oauth.retrieve_jwks,
        )
        self.token = to_raw_response_wrapper(
            oauth.token,
        )


class AsyncOAuthResourceWithRawResponse:
    def __init__(self, oauth: AsyncOAuthResource) -> None:
        self._oauth = oauth

        self.retrieve = async_to_raw_response_wrapper(
            oauth.retrieve,
        )
        self.grants = async_to_raw_response_wrapper(
            oauth.grants,
        )
        self.introspect = async_to_raw_response_wrapper(
            oauth.introspect,
        )
        self.register = async_to_raw_response_wrapper(
            oauth.register,
        )
        self.retrieve_authorize = async_to_raw_response_wrapper(
            oauth.retrieve_authorize,
        )
        self.retrieve_jwks = async_to_raw_response_wrapper(
            oauth.retrieve_jwks,
        )
        self.token = async_to_raw_response_wrapper(
            oauth.token,
        )


class OAuthResourceWithStreamingResponse:
    def __init__(self, oauth: OAuthResource) -> None:
        self._oauth = oauth

        self.retrieve = to_streamed_response_wrapper(
            oauth.retrieve,
        )
        self.grants = to_streamed_response_wrapper(
            oauth.grants,
        )
        self.introspect = to_streamed_response_wrapper(
            oauth.introspect,
        )
        self.register = to_streamed_response_wrapper(
            oauth.register,
        )
        self.retrieve_authorize = to_streamed_response_wrapper(
            oauth.retrieve_authorize,
        )
        self.retrieve_jwks = to_streamed_response_wrapper(
            oauth.retrieve_jwks,
        )
        self.token = to_streamed_response_wrapper(
            oauth.token,
        )


class AsyncOAuthResourceWithStreamingResponse:
    def __init__(self, oauth: AsyncOAuthResource) -> None:
        self._oauth = oauth

        self.retrieve = async_to_streamed_response_wrapper(
            oauth.retrieve,
        )
        self.grants = async_to_streamed_response_wrapper(
            oauth.grants,
        )
        self.introspect = async_to_streamed_response_wrapper(
            oauth.introspect,
        )
        self.register = async_to_streamed_response_wrapper(
            oauth.register,
        )
        self.retrieve_authorize = async_to_streamed_response_wrapper(
            oauth.retrieve_authorize,
        )
        self.retrieve_jwks = async_to_streamed_response_wrapper(
            oauth.retrieve_jwks,
        )
        self.token = async_to_streamed_response_wrapper(
            oauth.token,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr
from .rcs_agent_message_param import RcsAgentMessageParam

__all__ = ["MesssageRcsParams", "MmsFallback", "SMSFallback"]


class MesssageRcsParams(TypedDict, total=False):
    agent_id: Required[str]
    """RCS Agent ID"""

    agent_message: Required[RcsAgentMessageParam]

    messaging_profile_id: Required[str]
    """A valid messaging profile ID"""

    to: Required[str]
    """Phone number in +E.164 format"""

    mms_fallback: MmsFallback

    sms_fallback: SMSFallback

    type: Literal["RCS"]
    """Message type - must be set to "RCS" """

    webhook_url: str
    """The URL where webhooks related to this message will be sent."""


_MmsFallbackReservedKeywords = TypedDict(
    "_MmsFallbackReservedKeywords",
    {
        "from": str,
    },
    total=False,
)


class MmsFallback(_MmsFallbackReservedKeywords, total=False):
    media_urls: SequenceNotStr[str]
    """List of media URLs"""

    subject: str
    """Subject of the message"""

    text: str
    """Text"""


_SMSFallbackReservedKeywords = TypedDict(
    "_SMSFallbackReservedKeywords",
    {
        "from": str,
    },
    total=False,
)


class SMSFallback(_SMSFallbackReservedKeywords, total=False):
    text: str
    """Text (maximum 3072 characters)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .auto_recharge_prefs import (
    AutoRechargePrefsResource,
    AsyncAutoRechargePrefsResource,
    AutoRechargePrefsResourceWithRawResponse,
    AsyncAutoRechargePrefsResourceWithRawResponse,
    AutoRechargePrefsResourceWithStreamingResponse,
    AsyncAutoRechargePrefsResourceWithStreamingResponse,
)

__all__ = ["PaymentResource", "AsyncPaymentResource"]


class PaymentResource(SyncAPIResource):
    @cached_property
    def auto_recharge_prefs(self) -> AutoRechargePrefsResource:
        return AutoRechargePrefsResource(self._client)

    @cached_property
    def with_raw_response(self) -> PaymentResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return PaymentResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PaymentResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return PaymentResourceWithStreamingResponse(self)


class AsyncPaymentResource(AsyncAPIResource):
    @cached_property
    def auto_recharge_prefs(self) -> AsyncAutoRechargePrefsResource:
        return AsyncAutoRechargePrefsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncPaymentResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPaymentResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPaymentResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncPaymentResourceWithStreamingResponse(self)


class PaymentResourceWithRawResponse:
    def __init__(self, payment: PaymentResource) -> None:
        self._payment = payment

    @cached_property
    def auto_recharge_prefs(self) -> AutoRechargePrefsResourceWithRawResponse:
        return AutoRechargePrefsResourceWithRawResponse(self._payment.auto_recharge_prefs)


class AsyncPaymentResourceWithRawResponse:
    def __init__(self, payment: AsyncPaymentResource) -> None:
        self._payment = payment

    @cached_property
    def auto_recharge_prefs(self) -> AsyncAutoRechargePrefsResourceWithRawResponse:
        return AsyncAutoRechargePrefsResourceWithRawResponse(self._payment.auto_recharge_prefs)


class PaymentResourceWithStreamingResponse:
    def __init__(self, payment: PaymentResource) -> None:
        self._payment = payment

    @cached_property
    def auto_recharge_prefs(self) -> AutoRechargePrefsResourceWithStreamingResponse:
        return AutoRechargePrefsResourceWithStreamingResponse(self._payment.auto_recharge_prefs)


class AsyncPaymentResourceWithStreamingResponse:
    def __init__(self, payment: AsyncPaymentResource) -> None:
        self._payment = payment

    @cached_property
    def auto_recharge_prefs(self) -> AsyncAutoRechargePrefsResourceWithStreamingResponse:
        return AsyncAutoRechargePrefsResourceWithStreamingResponse(self._payment.auto_recharge_prefs)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["PortingOrderUserFeedbackParam"]


class PortingOrderUserFeedbackParam(TypedDict, total=False):
    user_comment: str
    """A comment related to the customer rating."""

    user_rating: int
    """
    Once an order is ported, cancellation is requested or the request is cancelled,
    the user may rate their experience
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["NumberOrderListParams", "Filter", "FilterCreatedAt", "Page"]


class NumberOrderListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[status], filter[created_at], filter[phone_numbers_count],
    filter[customer_reference], filter[requirements_met]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[size], page[number]
    """


class FilterCreatedAt(TypedDict, total=False):
    gt: str
    """Filter number orders later than this value."""

    lt: str
    """Filter number orders earlier than this value."""


class Filter(TypedDict, total=False):
    created_at: FilterCreatedAt
    """Filter number orders by date range."""

    customer_reference: str
    """Filter number orders via the customer reference set."""

    phone_numbers_count: str
    """Filter number order with this amount of numbers"""

    requirements_met: bool
    """Filter number orders by requirements met."""

    status: str
    """Filter number orders by status."""


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""
